package ch.std.jumpstartsecured;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertNotNull;

import java.util.HashSet;
import java.util.Set;

import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.context.SpringBootTest.WebEnvironment;
import org.springframework.boot.test.web.client.TestRestTemplate;
import org.springframework.test.context.ActiveProfiles;

import ch.std.jumpstartsecured.jpa.City;
import ch.std.jumpstartsecured.jpa.Role;
import ch.std.jumpstartsecured.jpa.User;
import ch.std.jumpstartsecured.repository.CityRepository;
import ch.std.jumpstartsecured.repository.UserRepository;

@SpringBootTest(webEnvironment = WebEnvironment.RANDOM_PORT)
@ActiveProfiles("test")
public class JumpstartApplicationJpaTests {

    @Autowired
	private TestRestTemplate restTemplate;
    
    @Autowired
    private UserRepository userRepository;
    @Autowired
    private CityRepository cityRepository;
    
	@BeforeEach
	public void setup() {
		User user = new User();
		user.setUsername("user");
		user.setPassword("{noop}password");
		Set<Role> userRoles = new HashSet<>();
		Role userRole = new Role();			
		userRole.setRole("USER");
		userRoles.add(userRole);
		user.setRoles(userRoles);
		userRepository.save(user);
		User admin = new User();
		admin.setUsername("admin");
		admin.setPassword("{noop}admin");
		Set<Role> adminRoles = new HashSet<>();
		Role adminRole = new Role();
		adminRole.setRole("ADMIN");
		adminRoles.add(adminRole);
		admin.setRoles(adminRoles);
		userRepository.save(admin);

		City city = new City();
		city.setName("Bern");
		this.cityRepository.save(city);
	}

	@AfterEach
	public void tearDown() {
		this.userRepository.deleteAll();
		this.cityRepository.deleteAll();
	}
	
    @Test
	public void contextLoads() {
	}
    
	@Test
	public void testCityAutoCompleteController() throws Exception {
		String url = "/rest/auto/citiesjpa?value=Bern";
		City[] cities = this.restTemplate.withBasicAuth("user",  "password").getForObject(url, City[].class);
		assertNotNull(cities);
		assertEquals(1,  cities.length);
		City city = cities[0];
		assertNotNull(city);
		assertEquals("Bern", city.getName());
	}
	
}
